
#include <stdio.h>
#include <stdlib.h>
#include "cell.h"
#include "rif.h"
#include "csi.h"

extern void	Error(/* char *msg */);

#define STR_LEN			256
#define ALLOC(type,size)	(type *)malloc(sizeof(type)*(size_t)(size))
#define CELL_BITS		(sizeof(Cell)*8)

int	main(argc,argv)
int	argc;
char	**argv;
{
	char	**path,line[STR_LEN],
		*mae="memory allocation error.",
		*bbp=" : bad box parameter.";
	int	Nx,Ny,Nz,lower,upper,z,y,x,nx,ny,idx,Sx,Sy,Sz,Bx,By,Bz,
		z2,Z,z1,y2,Y,y1,x2,X,x1;
	Cell	***cell,bit,**ptr,**slice;
	long	Pels=0L;

	if (argc!=5) Error("usage : fd_bc directory nameFile lower upper");

	path=CheckSliceImages(argv[1],argv[2],&Nx,&Ny,&Nz,NULL);

	lower=atoi(argv[3]);
	upper=atoi(argv[4]);

	if ((cell=ALLOC(Cell **,(Nz+CELL_BITS-1)/CELL_BITS))==NULL) Error(mae);

	for (z=0; z<Nz; z++) {
	    if ((bit=(Cell)(1<<(z%CELL_BITS)))==1) {
		if ((ptr=cell[z/CELL_BITS]=ALLOC(Cell *,Ny))==NULL) Error(mae);

		for (y=0; y<Ny; y++) {
		    if ((ptr[y]=ALLOC(Cell,Nx))==NULL) Error(mae);

		    for (x=0; x<Nx; x++) ptr[y][x]=0;
		}
	    }
	    ReadImageFile(path[z],&nx,&ny,NULL,&slice,NULL);

	    for (y=0; y<ny; y++) {
		for (x=0; x<nx; x++)
		    if (slice[y][x]>=lower && slice[y][x]<=upper) {
			++Pels; ptr[y][x]|=bit;
		    }

		free(slice[y]);
	    }
	    free(slice);
	}
	(void)fprintf(stderr,"%d\t%d\t%d\t%ld\n",Nx,Ny,Nz,Pels);

	if (!Pels) Error("no occupied pixel.");

	while (fgets(line,STR_LEN,stdin)!=NULL) {
	    for (idx=0; idx<STR_LEN; idx++) if (line[idx]=='\0') break;

	    if (idx==STR_LEN || line[idx-1]!='\n') Error("too long line.");

	    line[idx-1]='\0';

	    switch(sscanf(line,"%d %d %d %d %d %d",&Sx,&Sy,&Sz,&Bx,&By,&Bz)) {
		case 1 : Bz=By=Bx=0; Sz=Sy=Sx; break;
		case 3 : Bz=By=Bx=0; break;
		case 4 : Bz=Bx; By=Sz; Bx=Sy; Sz=Sy=Sx; break;
		case 6 : break;
		default: (void)fputs(line,stderr); Error(bbp);
	    }
	    if (Sx<1 || Sy<1 || Sz<1 || (Sx> Nx && Sy> Ny && Sz> Nz) ||
		Bx<0 || By<0 || Bz<0 ||  Bx>=Sx || By>=Sy || Bz>=Sz) {
		(void)fputs(line,stderr); Error(bbp);
	    }

Pels=0L;
for (z2=0, Z=(Bz)?0:1; z2<Nz; Z++) {
    z1=z2; z2=Bz+Sz*Z; if (z2>Nz) z2=Nz;
    for (y2=0, Y=(By)?0:1; y2<Ny; Y++) {
	y1=y2; y2=By+Sy*Y; if (y2>Ny) y2=Ny;
	for (x2=0, X=(Bx)?0:1; x2<Nx; X++) {
	    x1=x2; x2=Bx+Sx*X; if (x2>Nx) x2=Nx;

	    for (z=z1; z<z2; z++) {
		bit=(Cell)(1<<(z%CELL_BITS)); ptr=cell[z/CELL_BITS];

		for (y=y1; y<y2; y++)
		for (x=x1; x<x2; x++)
		    if (ptr[y][x]&bit) {
			++Pels; goto Continue;
		    }
	    }
Continue:;
	}
    }
}
(void)printf("%d\t%d\t%d\t%d\t%d\t%d\t%ld\n",Sx,Sy,Sz,Bx,By,Bz,Pels);

	}
	return 0;
}
